#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations
from binascii import hexlify
from .GeneratorUtils import GeneratorUtils
from .KeyDto import KeyDto
from .SignatureDto import SignatureDto

class CosignatureBuilder:
    """Cosignature attached to an aggregate transaction.

    Attributes:
        version: Version.
        signerPublicKey: Cosigner public key.
        signature: Cosigner signature.
    """

    def __init__(self, version: int, signerPublicKey: KeyDto, signature: SignatureDto):
        """Constructor.
        Args:
            version: Version.
            signerPublicKey: Cosigner public key.
            signature: Cosigner signature.
        """
        self.version = version
        self.signerPublicKey = signerPublicKey
        self.signature = signature

    @staticmethod
    def from_tuple(t):
        return CosignatureBuilder(t[0], KeyDto(t[1]), SignatureDto(t[2]))

    def as_tuple(self):
        return (self.version, self.signerPublicKey.key, self.signature.signature)

    @classmethod
    def load_from_binary(cls, payload: bytes) -> CosignatureBuilder:
        """Creates an instance of CosignatureBuilder from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of CosignatureBuilder.
        """
        bytes_ = bytes(payload)

        version = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 8))  # kind:SIMPLE
        bytes_ = bytes_[8:]
        signerPublicKey = KeyDto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[signerPublicKey.get_size():]
        signature = SignatureDto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[signature.get_size():]
        return CosignatureBuilder(version, signerPublicKey, signature)

    def get_version(self) -> int:
        """Gets version.
        Returns:
            Version.
        """
        return self.version

    def get_signer_public_key(self) -> KeyDto:
        """Gets cosigner public key.
        Returns:
            Cosigner public key.
        """
        return self.signerPublicKey

    def get_signature(self) -> SignatureDto:
        """Gets cosigner signature.
        Returns:
            Cosigner signature.
        """
        return self.signature

    def get_size(self) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        size = 0
        size += 8  # version
        size += self.signerPublicKey.get_size()
        size += self.signature.get_size()
        return size

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(self.get_version(), 8))  # kind:SIMPLE
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.signerPublicKey.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.signature.serialize())  # kind:CUSTOM
        return bytes_

    def __str__(self):
        """Returns nice representation.
        Returns:
            Printable string
        """
        result = '('
        result += '{}, '.format(self.version)
        result += '{}, '.format(hexlify(self.signerPublicKey.key).decode('utf-8'))
        result += '{}'.format(hexlify(self.signature.signature).decode('utf-8'))
        result += ')'
        return result
    # end of class



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations
from .GeneratorUtils import GeneratorUtils
from .ProofGammaDto import ProofGammaDto
from .ProofScalarDto import ProofScalarDto
from .ProofVerificationHashDto import ProofVerificationHashDto

class VrfProofBuilder:
    """Verfiable random function proof.

    Attributes:
        gamma: Gamma.
        verificationHash: Verification hash.
        scalar: Scalar.
    """

    def __init__(self, gamma: ProofGammaDto, verificationHash: ProofVerificationHashDto, scalar: ProofScalarDto):
        """Constructor.
        Args:
            gamma: Gamma.
            verificationHash: Verification hash.
            scalar: Scalar.
        """
        self.gamma = gamma
        self.verificationHash = verificationHash
        self.scalar = scalar


    @classmethod
    def load_from_binary(cls, payload: bytes) -> VrfProofBuilder:
        """Creates an instance of VrfProofBuilder from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of VrfProofBuilder.
        """
        bytes_ = bytes(payload)

        gamma = ProofGammaDto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[gamma.get_size():]
        verificationHash = ProofVerificationHashDto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[verificationHash.get_size():]
        scalar = ProofScalarDto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[scalar.get_size():]
        return VrfProofBuilder(gamma, verificationHash, scalar)

    def get_gamma(self) -> ProofGammaDto:
        """Gets gamma.
        Returns:
            Gamma.
        """
        return self.gamma

    def get_verification_hash(self) -> ProofVerificationHashDto:
        """Gets verification hash.
        Returns:
            Verification hash.
        """
        return self.verificationHash

    def get_scalar(self) -> ProofScalarDto:
        """Gets scalar.
        Returns:
            Scalar.
        """
        return self.scalar

    def get_size(self) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        size = 0
        size += self.gamma.get_size()
        size += self.verificationHash.get_size()
        size += self.scalar.get_size()
        return size

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.gamma.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.verificationHash.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.scalar.serialize())  # kind:CUSTOM
        return bytes_
    # end of class



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations
from typing import List
from .GeneratorUtils import GeneratorUtils
from .MosaicResolutionEntryBuilder import MosaicResolutionEntryBuilder
from .ReceiptBuilder import ReceiptBuilder
from .ReceiptTypeDto import ReceiptTypeDto
from .UnresolvedMosaicIdDto import UnresolvedMosaicIdDto

class MosaicResolutionStatementBuilder(ReceiptBuilder):
    """Binary layout for a mosaic resolution statement.

    Attributes:
        unresolved: Unresolved mosaic.
        resolutionEntries: Resolution entries.
    """

    def __init__(self, version: int, type: ReceiptTypeDto, unresolved: UnresolvedMosaicIdDto, resolutionEntries: List[MosaicResolutionEntryBuilder]):
        """Constructor.
        Args:
            version: Receipt version.
            type: Receipt type.
            unresolved: Unresolved mosaic.
            resolutionEntries: Resolution entries.
        """
        super().__init__(version, type)
        self.unresolved = unresolved
        self.resolutionEntries = resolutionEntries


    @classmethod
    def load_from_binary(cls, payload: bytes) -> MosaicResolutionStatementBuilder:
        """Creates an instance of MosaicResolutionStatementBuilder from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of MosaicResolutionStatementBuilder.
        """
        bytes_ = bytes(payload)
        superObject = ReceiptBuilder.load_from_binary(bytes_)
        bytes_ = bytes_[superObject.get_size():]

        unresolved = UnresolvedMosaicIdDto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[unresolved.get_size():]
        resolutionEntries: List[MosaicResolutionEntryBuilder] = []
        bytes_ = GeneratorUtils.load_from_binary(MosaicResolutionEntryBuilder, resolutionEntries, bytes_, len(bytes_))
        return MosaicResolutionStatementBuilder(superObject.version, superObject.type, unresolved, resolutionEntries)

    def get_unresolved(self) -> UnresolvedMosaicIdDto:
        """Gets unresolved mosaic.
        Returns:
            Unresolved mosaic.
        """
        return self.unresolved

    def get_resolution_entries(self) -> List[MosaicResolutionEntryBuilder]:
        """Gets resolution entries.
        Returns:
            Resolution entries.
        """
        return self.resolutionEntries

    def get_size(self) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        size = super().get_size()
        size += self.unresolved.get_size()
        for _ in self.resolutionEntries:
            size += _.get_size()
        return size

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, super().serialize())
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.unresolved.serialize())  # kind:CUSTOM
        for _ in self.resolutionEntries: # kind:ARRAY|FILL_ARRAY
            bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, _.serialize())
        return bytes_
    # end of class
